VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "FTP"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

'
'  Copyright  1998-2002 Dev Ashish & Terry Kreft, All Rights Reserved
'  The Access Web       [ http://www.mvps.org/access ]
'  Terms of Use:           [ http://www.mvps.org/access/terms.htm ]
'  Dev Ashish (dev@mvps.org) ; Terry Kreft (terry.kreft@mps.co.uk)
'

' Event raised when a file is being downloaded
Public Event FileProgress(ByVal Factor As Single)

Private m_hInetOpen As Long
Private m_hConnect As Long
Private m_hFTP As Long
Private m_tURLInfo As ITL_URL_COMPONENTS
Private m_lngSize As Long
Private m_Folders As Folders
Private m_Files As Files
Private Const SEARCH_ARG = "*.*"
Private Const ERR_GENERIC = vbObjectError + 5555

Private Enum itlStatusContext
    scFtpCreateDirectory = 100
    scFtpDeleteFile
    scFtpFindFirstFile
    scFtpGetCurrentDirectory
    scFtpGetFile
    scFtpOpenFile
    scFtpPutFile
    scFtpRemoveDirectory
    scFtpRenameFile
    scFtpSetCurrentDirectory
    scGopherFindFirstFile
    scGopherOpenFile
    scHttpEndRequest
    scHttpOpenRequest
    scHttpSendRequestEx
    scInternetConnect
    scInternetOpenUrl
    scInternetReadFileEx
End Enum

Private m_strExecContext As String
Private m_blnIsErr As Boolean
Private Const CLS_NAME = "FTP -- "

' Whether to raise WinInet errors or not
Public RaiseInternetAPIErrors As Boolean

Public Sub CreateDirectory(ByVal Path As String)
Attribute CreateDirectory.VB_Description = "Creates a folder at the specified location on the remote server"
'
'   Creates a folder at the specified location on the remote server
'
On Error GoTo ErrHandler

    m_strExecContext = fGetExecContext("CreateDirectory")

    m_blnIsErr = fAssertInetErr( _
        (FtpCreateDirectory(m_hConnect, Path) = 0), _
        m_strExecContext, RaiseInternetAPIErrors)
    
    Exit Sub
ErrHandler:
   With Err
       .Raise .Number, .source, .Description, .HelpFile, .HelpContext
   End With
End Sub

Public Sub Delete(ByVal FileName As String)
Attribute Delete.VB_Description = "Removes a file from the remote server"
'
'   Removes a file from the remote server
'
On Error GoTo ErrHandler
    
    m_strExecContext = fGetExecContext("Delete")
    
    If (Len(FileName) = 0) Then
        With Err
            .Raise ITL_ERROR_FILE_NOT_FOUND, "FTP::Delete", _
                        "The system cannot find the path specified."
        End With
    End If
    
    m_blnIsErr = fAssertInetErr( _
        (FtpDeleteFile(m_hConnect, FileName) = 0), _
        m_strExecContext, RaiseInternetAPIErrors)
        
    Exit Sub
ErrHandler:
   With Err
       .Raise .Number, .source, .Description, .HelpFile, .HelpContext
   End With
End Sub

Public Sub Upload( _
                Optional ByVal TransferType As itlTransferFlags = ITL_INTERNET_FLAG_TRANSFER_BINARY, _
                Optional ByVal LocalFileName As String, _
                Optional ByVal UploadURL As String, _
                Optional OverWriteIfPresentOnServer As Boolean, _
                Optional ByVal hWnd As Long, _
                Optional ByVal DialogTitle As String, _
                Optional ByVal Filter As String = "All Files|*.*", _
                Optional ByVal StartInDirectory As String)
Attribute Upload.VB_Description = "Uploads the specified local file to the server on the FTP protocol"
'
' Uploads the specified local file to the server on the FTP protocol
'
' Arguments:
'       TransferType                                      [ Optional ]               Default Value = ITL_INTERNET_FLAG_TRANSFER_BINARY
'               How to transfer the file to the ftp server (Binary mode or ASCII mode)
'
'       LocalFileName                                   [ Optional ]               Default Value = vbNullString
'               Which local file to transfer to the server; if not set, the class will prompt you with a Windows Open dialog
'
'       UploadURL                                     [ Optional ]                Default Value = vbNullString
'               The path where the file to be transferred will reside
'
'       OverWriteIfPresentOnServer              [ Optional ]               Default Value = false
'               Whether to overwrite the file automatically on the server or not
'
'       hWnd                                              [ Optional ]               Default Value = 0
'               Handle to the calling form (for use with the GetOpenFileName call, (should it be necessary))
'
'       DialogTitle                               [ Optional ]                      Default Value = vbNullString
'               Title of the GetOpenFileName dialog (should it be necessary)
'
'       Filter                                       [ Optional ]                       Default Value = "All Files|*.*"
'                Mask to use with the GetOpenFileName call  (should it be necessary)
'
'       StartInDirectory                        [ Optional ]                      Default Value = vbNullString
'               Sets the current folder in the GetOpenFileName call (should it be necessary)
'
On Error GoTo ErrHandler
Dim hFile As Long
Dim lngSize As Long, i As Long
Dim pBuffer() As Byte, lngBytesWritten As Long
Dim lngFrags As Long, lngLeftOver As Long

    m_strExecContext = fGetExecContext("Upload")

    If (Len(LocalFileName) = 0) Then
        ' If no file specified, then prompt the user
        LocalFileName = fGetFileName(vbNullString, hWnd, _
                                                    DialogTitle, Filter, vbNullString, _
                                                    StartInDirectory, True)
    End If

    If (Len(LocalFileName) = 0) Then Err.Raise ERR_GENERIC

    ' Establish a connection to the file
    m_hFTP = FtpOpenFile(m_hConnect, _
                                UploadURL, _
                                GENERIC_WRITE, _
                                TransferType Or ITL_INTERNET_FLAG_RELOAD, _
                                0)

    ' Bail out if the open was unsuccessful
    m_blnIsErr = fAssertInetErr( _
                                (m_hFTP = 0), _
                                m_strExecContext, _
                                RaiseInternetAPIErrors)
    If (m_blnIsErr) Then Err.Raise ERR_GENERIC
    

    ' Save the local file
    hFile = FreeFile
    Open LocalFileName For Binary Access Read Shared As hFile
    lngSize = LOF(hFile)
    ReDim pBuffer(MAX_CHUNK - 1)
    lngFrags = lngSize \ MAX_CHUNK
    lngLeftOver = lngSize Mod MAX_CHUNK

    For i = 1 To lngFrags
        Call ZeroMem(pBuffer(0), MAX_CHUNK)
        
        Get hFile, , pBuffer
        
        m_blnIsErr = fAssertInetErr( _
                                    (InternetWriteFile(m_hFTP, pBuffer(0), MAX_CHUNK, lngBytesWritten) = 0), _
                                    m_strExecContext, _
                                    RaiseInternetAPIErrors)
        If (m_blnIsErr) Then Err.Raise ERR_GENERIC
        
        ' Notify client of progress
        RaiseEvent FileProgress(Format$((i * MAX_CHUNK / lngSize) * 100, DECIMAL_FORMAT))
    Next
    
    ' write out the remaining bytes
    If (lngLeftOver) Then
        Call ZeroMem(pBuffer(0), MAX_CHUNK)
        
        Get hFile, , pBuffer
        
        m_blnIsErr = fAssertInetErr( _
                                (InternetWriteFile(m_hFTP, pBuffer(0), lngLeftOver, lngBytesWritten) = 0), _
                                m_strExecContext, RaiseInternetAPIErrors)
        If (m_blnIsErr) Then Err.Raise ERR_GENERIC
            
        RaiseEvent FileProgress(100)
    End If

    Close #hFile
    Call InternetCloseHandle(m_hFTP)
    Exit Sub
ErrHandler:
    Close #hFile
    Call InternetCloseHandle(m_hFTP)
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
    End With
End Sub

Public Sub Download( _
                Optional ByVal TransferType As itlTransferFlags = ITL_INTERNET_FLAG_TRANSFER_BINARY, _
                Optional ByVal URL As String, _
                Optional ByVal OverWriteIfPresent As Boolean, _
                Optional ByVal LocalFileName As String, _
                Optional ByVal hWnd As Long, _
                Optional ByVal DialogTitle As String, _
                Optional ByVal Filter As String = "All Files|*.*", _
                Optional ByVal StartInDirectory As String)
Attribute Download.VB_Description = "Downloads a file from the server on the FTP protocol and saves it locally"
'
' Downloads a file from the server on the FTP protocol and saves it locally
'
' Arguments:
'       TransferType                                      [ Optional ]               Default Value = ITL_INTERNET_FLAG_TRANSFER_BINARY
'               How to transfer the file to the ftp server (Binary mode or ASCII mode)
'
'       URL                                                    [ Optional ]               Default Value = vbNullString
'               Which remote file to transfer to the local machine
'
'
'       OverWriteIfPresentOnServer              [ Optional ]               Default Value = false
'               Whether to overwrite the local file automatically or not
'
'       LocalFileName                                  [ Optional ]             Default Value = vbNullString
'               Which file name to use locally; if not set, prompts the user with GetSaveFileName dialog
'
'       hWnd                                              [ Optional ]               Default Value = 0
'               Handle to the calling form (for use with the GetSaveFileName call, (should it be necessary))
'
'       DialogTitle                               [ Optional ]                      Default Value = vbNullString
'               Title of the GetSaveFileName dialog (should it be necessary)
'
'       Filter                                       [ Optional ]                       Default Value = "All Files|*.*"
'                Mask to use with the GetSaveFileName call  (should it be necessary)
'
'       StartInDirectory                        [ Optional ]                      Default Value = vbNullString
'               Sets the current folder in the GetSaveFileName call (should it be necessary)
'
                
On Error GoTo ErrHandler
Dim hFile As Long
Dim abytData() As Byte
Dim lngBytesRead As Long, lngBytesWritten As Long
Dim lngTotalBytesWritten As Long, lngRet As Long

    m_strExecContext = fGetExecContext("Download")

    ' Determine whether or not to call GetSaveFileName
    ' and if the specified name already exists or not
    If (Len(LocalFileName) = 0) Then
        LocalFileName = fGetFileName(vbNullString, hWnd, DialogTitle, Filter, vbNullString, StartInDirectory, False)
    Else
        If (Len(Dir$(LocalFileName, vbNormal))) Then
            If (OverWriteIfPresent) Then
                'Do nothing
            Else
                With Err
                    .Raise ITL_ERROR_FILE_ALREADY_EXISTS, "FTP::Download", _
                        "The file specified, '" & LocalFileName & "', " & _
                            "already exists." & vbCrLf & _
                            "Either delete the file before calling the 'Download' method, " & _
                            "or set the OverWriteIfPresent argument in the method call to True."
                End With
            End If
        End If
    End If
    
    If (Len(LocalFileName) = 0) Then Err.Raise ERR_GENERIC
    
    ' Open the remote file for writing
    m_hFTP = FtpOpenFile(m_hConnect, _
                                URL, _
                                GENERIC_READ, _
                                TransferType Or ITL_INTERNET_FLAG_DONT_CACHE, _
                                0)
    
    m_blnIsErr = fAssertInetErr((m_hFTP = 0), m_strExecContext, RaiseInternetAPIErrors)
    If (m_blnIsErr) Then Err.Raise ERR_GENERIC
    
    ' Get the size of the file
    m_lngSize = fGetFTPFileSize(URL)
    
    If (m_lngSize = 0) Then m_lngSize = 100
    
    'Create the destination file
    hFile = CreateFile(LocalFileName, _
                GENERIC_READ Or GENERIC_WRITE, _
                0, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0&)
                
    m_blnIsErr = fAssertInetErr((hFile = 0), m_strExecContext, RaiseInternetAPIErrors)
    If (m_blnIsErr) Then Err.Raise ERR_GENERIC
    
    Do
        ReDim abytData(MAX_CHUNK - 1)
        lngRet = InternetReadFile( _
                        m_hFTP, _
                        abytData(0), _
                        MAX_CHUNK, _
                        lngBytesRead)
        m_blnIsErr = fAssertInetErr((lngRet = 0), m_strExecContext, RaiseInternetAPIErrors)
        If (m_blnIsErr) Then Err.Raise ERR_GENERIC
        
        If (lngBytesRead) Then
            m_blnIsErr = fAssertInetErr( _
                (WriteFile(hFile, abytData(0), lngBytesRead, lngBytesWritten, 0) = 0), _
                m_strExecContext, RaiseInternetAPIErrors)
                
            lngTotalBytesWritten = lngTotalBytesWritten + lngBytesWritten
            RaiseEvent FileProgress(Format$((lngTotalBytesWritten / m_lngSize) * 100, DECIMAL_FORMAT))
        End If
    Loop Until lngRet <> 0 And lngBytesRead = 0
    
    Call FlushFileBuffers(hFile)
    Call CloseHandle(hFile)
    Call InternetCloseHandle(m_hFTP)
    Exit Sub
ErrHandler:
    Call FlushFileBuffers(hFile)
    Call CloseHandle(hFile)
    Call InternetCloseHandle(m_hFTP)
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
    End With
End Sub

Public Function FileDialog( _
                                ByVal InitialFileName As String, _
                                ByVal DialogTitle As String, _
                                ByVal Filter As String, _
                                ByVal DefaultExtension As String, _
                                ByVal StartInDirectory As String, _
                                ByVal OpenMode As Boolean, _
                                Optional ByVal hWnd As Long) _
                                As String
Attribute FileDialog.VB_Description = "Wrapper to GetOpenFileName/GetSaveAsFileName API"
'
'   Wrapper to GetOpenFileName API
'
On Error GoTo ErrHandler
        
    m_strExecContext = fGetExecContext("FileDialog")
        
    FileDialog = fGetFileName(InitialFileName, hWnd, DialogTitle, Filter, _
                                            DefaultExtension, StartInDirectory, OpenMode)
    
    Exit Function
ErrHandler:
   With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Function

Public Sub Rename(ByVal OldFileName As String, ByVal NewFileName As String)
Attribute Rename.VB_Description = "Renames a file on the remote ftp server"
'
'   Renames a file on the remote server
'
On Error GoTo ErrHandler
    
    m_strExecContext = fGetExecContext("Rename")
    
    m_blnIsErr = fAssertInetErr( _
            (FtpRenameFile(m_hConnect, OldFileName, NewFileName) = 0), _
            m_strExecContext, RaiseInternetAPIErrors)
    
    Exit Sub
ErrHandler:
   With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Sub

Public Sub ChangeDirectory(ByVal NewPath As String, Optional ByVal AutoCreateMissingFolders As Boolean)
Attribute ChangeDirectory.VB_Description = "Changes the current path on the ftp server"
'
'   Changes the current path on the ftp server
'
On Error GoTo ErrHandler
Dim lngRet As Long
Dim astrNames() As String, i As Integer
Const vbKeySlash = "/"

    m_strExecContext = fGetExecContext("ChangeDirectory")

    If (Len(NewPath) = 1 And StrComp(NewPath, vbKeySlash, vbTextCompare) = 0) Then
        ReDim astrNames(0)
        astrNames(0) = vbKeySlash
    Else
        astrNames = Split(NewPath, vbKeySlash)
    End If
    
    For i = LBound(astrNames) To UBound(astrNames)
        If (Len(astrNames(i))) Then
            lngRet = FtpSetCurrentDirectory(m_hConnect, astrNames(i) & vbKeySlash)
            If (lngRet = 0) Then
                If (Err.Number = ITL_ERROR_INTERNET_EXTENDED_ERROR) Then
                    If (AutoCreateMissingFolders) Then
                        m_blnIsErr = fAssertInetErr( _
                                (FtpCreateDirectory(m_hConnect, astrNames(i)) = 0), _
                                m_strExecContext, RaiseInternetAPIErrors)
                        If (m_blnIsErr) Then Err.Raise ERR_GENERIC
                    Else
                        m_blnIsErr = fAssertInetErr( _
                                (True), _
                                m_strExecContext, RaiseInternetAPIErrors)
                        If (m_blnIsErr) Then Err.Raise ERR_GENERIC
                    End If
                Else
                    m_blnIsErr = fAssertInetErr( _
                                (True), _
                                m_strExecContext, RaiseInternetAPIErrors)
                    If (m_blnIsErr) Then Err.Raise ERR_GENERIC
                End If
            End If
        End If
    Next
    
    Exit Sub
ErrHandler:
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Sub

Public Property Get CurrentDirectory() As String
Attribute CurrentDirectory.VB_Description = "Returns the current directory on the remote server"
'
'   Returns the current directory on the remote server
'
On Error GoTo ErrHandler
Dim strPath As String
Dim lngRet As Long
Dim lngLen As Long

    m_strExecContext = fGetExecContext("CurrentDirectory")

    strPath = String$(MAX_PATH, vbNullChar)
    lngLen = Len(strPath)

    m_blnIsErr = fAssertInetErr( _
        (FtpGetCurrentDirectory(m_hConnect, strPath, lngLen) = 0), _
        m_strExecContext, RaiseInternetAPIErrors)

    CurrentDirectory = Left$(strPath, lngLen)

    Exit Property
ErrHandler:
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Property

Public Property Get Folders() As Folders
Attribute Folders.VB_Description = "Returns all the sub-folders found on the ftp server's working directory"
'
'   Returns all the sub-folders found on the ftp server's working directory
'
    m_strExecContext = fGetExecContext("Folders")
    If m_Folders Is Nothing Then Set m_Folders = New Folders
    Set Folders = m_Folders
End Property

Public Property Get Files() As Files
Attribute Files.VB_Description = "Returns all the files found on the ftp server's working directory"
'
'   Returns all the files found on the ftp server's working directory
'
    m_strExecContext = fGetExecContext("Files")
    If m_Files Is Nothing Then Set m_Files = New Files
    Set Files = m_Files
End Property

Public Sub EnumerateFolder(ByVal Path As String)
'
'   Recursively enumerates the specified path on the server
'   to populate the internal files and folders collection
'
On Error GoTo ErrHandler
    
    m_strExecContext = fGetExecContext("EnumerateFolder")
    
    Call sEnumerateFolder(Path)

    Exit Sub
ErrHandler:
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
    End With
End Sub

Public Sub Disconnect()
Attribute Disconnect.VB_Description = "Resets the WinInet environment"
'
'   Resets the WinInet environment
'
    m_strExecContext = fGetExecContext("Disconnect")
    Call Me.InternetStop
End Sub

Public Function FolderDialog( _
                                ByVal hWnd As Long, _
                                ByVal DialogTitle As String, _
                                ByVal CurrentDirectory As String) _
                                As String
Attribute FolderDialog.VB_Description = "Allows user to select local folder to save the remote file in"
On Error GoTo ErrHandler
'
'   Allows user to select local folder to save the remote file in
'
    m_strExecContext = fGetExecContext("FolderDialog")
    
    FolderDialog = fGetNewFolderName(hWnd, DialogTitle, CurrentDirectory)
    
    Exit Function
ErrHandler:
   With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Function

Public Sub InternetStop()
Attribute InternetStop.VB_Description = "Reclaims all open resources being used by the API functions"
'
'   Reclaims all open resources being used by the API functions
'
On Error GoTo ErrHandler

    m_strExecContext = fGetExecContext("InternetStop")
    
    Call InternetCloseHandle(m_hFTP)
    Call InternetCloseHandle(m_hConnect)
    Call InternetCloseHandle(m_hInetOpen)

    m_hInetOpen = 0
    m_hFTP = 0
    m_hConnect = 0

    Exit Sub
ErrHandler:
   With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Sub

Private Function fGetFTPFileSize(strFile As String) As Long
'
'   Returns the size of the specified file
'
On Error Resume Next
    fGetFTPFileSize = m_Files.Item(strFile).Size
End Function

Private Static Sub sEnumerateFolder(strFolderName As String, Optional strSearchArg As String = SEARCH_ARG)
'
'   Recursively enumerates the specified path on the server
'   to populate the internal files and folders collection
'
On Error GoTo ErrHandler
Dim tDirInfo As WIN32_FIND_DATA
Dim hDir As Long, lngRet As Long

    ' Find the first file
    hDir = FtpFindFirstFile(m_hConnect, strSearchArg, tDirInfo, ITL_INTERNET_FLAG_RESYNCHRONIZE, 0)
    
    If Not (CBool(hDir)) Then
        
        If (Err.LastDllError <> ITL_ERROR_NO_MORE_FILES) Then
            m_blnIsErr = fAssertInetErr(True, m_strExecContext, RaiseInternetAPIErrors)
            If (m_blnIsErr) Then Err.Raise ERR_GENERIC
        Else
            ' no files
            Call m_Files.ClearCollection
            Call m_Folders.ClearCollection
        End If
    Else
        
        Set m_Folders = New Folders
        Set m_Files = New Files
        If (tDirInfo.dwFileAttributes = ITL_FILE_ATTRIBUTE_DIRECTORY) Then
            m_Folders.Add fTrimNull(tDirInfo.cFileName)
        Else
            m_Files.Add tDirInfo
        End If
        
        lngRet = InternetFindNextFile(hDir, tDirInfo)
        m_blnIsErr = fAssertInetErr(lngRet = 0 And (Err.LastDllError <> ITL_ERROR_NO_MORE_FILES), m_strExecContext, RaiseInternetAPIErrors)
        If (m_blnIsErr) Then Err.Raise ERR_GENERIC
        
        ' Keep retrieving file names until no more files are found
        Do While (lngRet > 0 And Err.LastDllError <> ITL_ERROR_NO_MORE_FILES)
            With tDirInfo
                If (.dwFileAttributes = ITL_FILE_ATTRIBUTE_DIRECTORY) Then
                    m_Folders.Add fTrimNull(.cFileName)
                Else
                    m_Files.Add tDirInfo
                End If
            End With
            
            lngRet = InternetFindNextFile(hDir, tDirInfo)
            m_blnIsErr = fAssertInetErr((lngRet = 0) And (Err.LastDllError <> ITL_ERROR_NO_MORE_FILES), _
                                        m_strExecContext, RaiseInternetAPIErrors)
            If (m_blnIsErr) Then Err.Raise ERR_GENERIC
            
        Loop
    End If
    Call InternetCloseHandle(hDir)
    Exit Sub
ErrHandler:
    Call InternetCloseHandle(hDir)
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
    End With
End Sub

Public Sub Connect( _
                    ByVal URL As String, _
                    Optional ByVal UsePassiveMode As Boolean, _
                    Optional ByVal UserName As String, _
                    Optional ByVal Password As String)
Attribute Connect.VB_Description = "Establish a connection to the remote server"
'
'   Establish a connection to the remote server
'
On Error GoTo ErrHandler
Const FTP_LEADER_CHARS = "FTP://"
Dim lngFlags As Long
Dim lngSC As Long

    m_strExecContext = fGetExecContext("Connect")

    If (m_hInetOpen = 0) Then
        Call Me.InternetStart
    End If
    
    If (m_hConnect) Then Call InternetCloseHandle(m_hConnect)
    
    If (StrComp(Left$(URL, Len(FTP_LEADER_CHARS)), FTP_LEADER_CHARS, vbTextCompare)) <> 0 Then
        URL = FTP_LEADER_CHARS & URL
    End If
    
    If (fParseURL(m_tURLInfo, URL)) Then
        If (Len(m_tURLInfo.lpszHostName) = 0) Then
            Err.Raise itlErrors.ITL_ERROR_BAD_URL, "FTP::Connect", fInetError(Err.LastDllError)
        End If
    Else
        Err.Raise itlErrors.ITL_ERROR_UNKNOWN, "FTP::Connect", _
            "Unknown error occurred while trying to connect to the specified location."
    End If
    
    If (UsePassiveMode) Then lngFlags = ITL_INTERNET_FLAG_PASSIVE
   
    m_hConnect = InternetConnect( _
                            m_hInetOpen, _
                            m_tURLInfo.lpszHostName, _
                            INTERNET_INVALID_PORT_NUMBER, _
                            UserName, _
                            Password, _
                            INTERNET_SERVICE_FTP, _
                            lngFlags, _
                            lngSC)
    m_blnIsErr = fAssertInetErr((m_hConnect = 0), "FTP::Connect", RaiseInternetAPIErrors)
    If (m_blnIsErr) Then Err.Raise ERR_GENERIC
    
    Exit Sub
ErrHandler:
    With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
    End With
End Sub

Public Sub InternetStart( _
                        Optional AccessType As itlInternetOpenFlags = ITL_INTERNET_OPEN_TYPE_PRECONFIG, _
                        Optional ProxyName As String, _
                        Optional ProxyBypass As String, _
                        Optional Flags As itlOpenFlags)
Attribute InternetStart.VB_Description = "Set up the class to make WinInet calls"
'
'   Set up the class to make WinInet calls
'
On Error GoTo ErrHandler

    m_strExecContext = fGetExecContext("InternetStart")
    
    m_hInetOpen = InternetOpen( _
                                LoadResString(ITL_USER_AGENT), _
                                AccessType, _
                                ProxyName, _
                                ProxyBypass, _
                                Flags)
    
    m_blnIsErr = fAssertInetErr((m_hInetOpen = 0), m_strExecContext, RaiseInternetAPIErrors)
    If (m_blnIsErr) Then Err.Raise ERR_GENERIC

   Exit Sub
ErrHandler:
   With Err
        If (.Number <> ERR_GENERIC) Then
            .Raise .Number, .source, .Description, .HelpFile, .HelpContext
        End If
   End With
End Sub


Private Function fGetExecContext(strProcName As String) As String
    fGetExecContext = CLS_NAME & strProcName
End Function
