Attribute VB_Name = "modCommon"
Option Explicit

'
'  Copyright  1998-2002 Dev Ashish & Terry Kreft, All Rights Reserved
'  The Access Web       [ http://www.mvps.org/access ]
'  Terms of Use:           [ http://www.mvps.org/access/terms.htm ]
'  Dev Ashish (dev@mvps.org) ; Terry Kreft (terry.kreft@mps.co.uk)
'

Private Type SYSTEMTIME
  wYear As Integer
  wMonth As Integer
  wDayOfWeek As Integer
  wDay As Integer
  wHour As Integer
  wMinute As Integer
  wSecond As Integer
  wMilliseconds As Integer
End Type

Private Type TIME_ZONE_INFORMATION
  Bias As Long
  StandardName(0 To 63) As Byte
  StandardDate As SYSTEMTIME
  StandardBias As Long
  DaylightName(0 To 63) As Byte
  DaylightDate As SYSTEMTIME
  DaylightBias As Long
End Type

Private Declare Function FileTimeToSystemTime _
    Lib "kernel32" _
    (lpFileTime As FILETIME, _
    lpSystemTime As SYSTEMTIME) _
    As Long
    
Private Declare Function GetTimeFormat _
    Lib "kernel32" Alias "GetTimeFormatA" _
    (ByVal Locale As Long, _
    ByVal dwFlags As Long, _
    lpTime As SYSTEMTIME, _
    ByVal lpFormat As Long, _
    ByVal lpTimeStr As String, _
    ByVal cchTime As Long) _
    As Long
 
Private Declare Function GetDateFormat _
    Lib "kernel32" Alias "GetDateFormatA" _
    (ByVal Locale As Long, _
    ByVal dwFlags As Long, _
    lpDate As SYSTEMTIME, _
    ByVal lpFormat As Long, _
    ByVal lpDateStr As String, _
    ByVal cchDate As Long) _
    As Long
     
Private Declare Function GetTimeZoneInformation _
    Lib "kernel32" _
    (lpTimeZoneInformation As TIME_ZONE_INFORMATION) _
    As Long
     
     
Private Type DllVersionInfo
   cbSize As Long
   dwMajorVersion As Long
   dwMinorVersion As Long
   dwBuildNumber As Long
   dwPlatformID As Long
End Type

Private Declare Function DllGetVersion _
    Lib "Shlwapi.dll" _
  (dwVersion As DllVersionInfo) _
  As Long
     
Private Const LOCALE_SYSTEM_DEFAULT = &H800
Private Const LOCALE_USER_DEFAULT = &H400
'  do not use user overrides
Private Const LOCALE_NOUSEROVERRIDE = &H80000000
'  use the system ACP
Private Const LOCALE_USE_CP_ACP = &H40000000
Private Const TIME_ZONE_ID_INVALID = &HFFFFFFFF
Private Const TIME_ZONE_ID_UNKNOWN = 0
Private Const TIME_ZONE_ID_STANDARD = 1
Private Const TIME_ZONE_ID_DAYLIGHT = 2

Public Function fParseURL(tURLInfo As ITL_URL_COMPONENTS, ByVal strURL As String) As Boolean
Dim lngLen As Long, strBuffer As String
Dim lngRet As Long, strURLLocal As String

    strBuffer = String$(MAX_BUFFER, 0)
    lngLen = Len(strBuffer)
    
    With tURLInfo
        .lpszScheme = String$(MAX_BUFFER, 0)
        .dwSchemeLength = MAX_BUFFER
        .nScheme = INTERNET_SCHEME_UNKNOWN
        .lpszHostName = String$(MAX_BUFFER, 0)
        .dwHostNameLength = MAX_BUFFER
        .nPort = 0
        .lpszUserName = String$(MAX_BUFFER, 0)
        .dwUserNameLength = MAX_BUFFER
        .lpszPassword = String$(MAX_BUFFER, 0)
        .dwPasswordLength = MAX_BUFFER
        .lpszUrlPath = String$(MAX_BUFFER, 0)
        .dwUrlPathLength = MAX_BUFFER
        .lpszExtraInfo = String$(MAX_BUFFER, 0)
        .dwExtraInfoLength = MAX_BUFFER
        .dwStructSize = Len(tURLInfo)
    End With
    lngRet = InternetCrackUrl( _
                    strURL, Len(strURL), _
                    ITL_ICU_ESCAPE, _
                    tURLInfo)
    If (lngRet) Then
        lngLen = Len(tURLInfo.lpszExtraInfo)
        If (lngLen) Then
            lngRet = InternetCanonicalizeUrl(tURLInfo.lpszExtraInfo, strBuffer, lngLen, ITL_ICU_ENCODE_SPACES_ONLY)
            If (lngRet) Then
                tURLInfo.lpszExtraInfo = Mid$(strBuffer, 1, lngLen)
            End If
        End If
        With tURLInfo
            .lpszExtraInfo = fTrimNull(.lpszExtraInfo)
            .lpszHostName = fTrimNull(.lpszHostName)
            .lpszPassword = fTrimNull(.lpszPassword)
            .lpszScheme = fTrimNull(.lpszScheme)
            .lpszUrlPath = fTrimNull(.lpszUrlPath)
            .lpszUserName = fTrimNull(.lpszUserName)
        End With
        fParseURL = True
  End If
End Function


Public Function fAPIErr(ByVal lngErr As Long) As String
Dim strMsg As String
Dim lngRet As Long
    strMsg = String$(1024, 0)
    lngRet = FormatMsgLong(FORMAT_MESSAGE_FROM_SYSTEM, 0, _
                    lngErr, 0&, strMsg, Len(strMsg), ByVal 0&)
    If (lngRet) Then
        fAPIErr = Left$(strMsg, lngRet)
    End If
End Function


Public Function fInetError(ByVal lngErrCode As Long) As String
Dim strMsg As String
Dim lngRet As Long
Dim hModule As Long
Dim lngLen As Long, strBuffer As String
  
    If (lngErrCode = ITL_ERROR_INTERNET_EXTENDED_ERROR) Then
        Call InternetGetLastResponse(lngErrCode, vbNullString, lngLen)
        strBuffer = String$(lngLen + 1, 0)
        Call InternetGetLastResponse(lngErrCode, strBuffer, lngLen)
        fInetError = strBuffer
    Else
         strMsg = String$(1024, 0)
        hModule = LoadLibraryEx("WinInet.dll", 0, LOAD_LIBRARY_AS_DATAFILE)
        lngRet = FormatMsgLong(FORMAT_MESSAGE_FROM_SYSTEM Or _
                    FORMAT_MESSAGE_IGNORE_INSERTS Or _
                    FORMAT_MESSAGE_MAX_WIDTH_MASK Or _
                    FORMAT_MESSAGE_FROM_HMODULE, hModule, _
                    lngErrCode, 0&, strMsg, Len(strMsg), ByVal 0&)
        If (lngRet) Then
            fInetError = Left$(strMsg, lngRet)
        End If
        Call FreeLibrary(hModule)
    End If
End Function

Public Static Function fTrimNull(ByVal strIn As String) As String
Dim intPos As Integer
    intPos = InStr(1, strIn, vbNullChar, vbTextCompare)
    If (intPos) Then
        fTrimNull = Left$(strIn, intPos - 1)
    Else
        fTrimNull = strIn
    End If
End Function

Public Function fGetFileName(ByVal InitFileName As String, _
                                ByVal hWnd As Long, _
                                ByVal strDialogTitle As String, _
                                ByVal strFilter As String, _
                                ByVal strDefaultExtension As String, _
                                ByVal strStartInDirectory As String, _
                                ByVal blnOpenMode As Boolean) _
                                As String
Dim clsDialog As CDialog
  
    If (hWnd = 0) Then hWnd = GetForegroundWindow()
    If (Len(strStartInDirectory) = 0) Then strStartInDirectory = CurDir()
    If (Len(strDialogTitle) = 0) Then
        If (blnOpenMode) Then
            strDialogTitle = "Open file"
        Else
            strDialogTitle = "Save file as"
        End If
    End If
      
    Set clsDialog = New CDialog

    With clsDialog
        .hWnd = hWnd
        .ModeOpen = blnOpenMode
        .Filter = strFilter
        .FileName = InitFileName
        .StartDir = strStartInDirectory
        .Title = strDialogTitle
        If (Len(strDefaultExtension)) Then
            .DefaultExtension = strDefaultExtension
        End If
        fGetFileName = .Action
    End With
    Set clsDialog = Nothing
End Function

Public Function fGetNewFolderName(hWnd As Long, strDialogTitle As String, strCurrentDir As String) As String
Dim clsDialog As CDialog
    Set clsDialog = New CDialog
    fGetNewFolderName = clsDialog.BrowseFolder(strDialogTitle, hWnd, strCurrentDir)
    Set clsDialog = Nothing
End Function

Public Function fGetFileTime(lpFileData As WIN32_FIND_DATA, intTimeType As Integer) As String
'   Returns a particular FileTime associated with a file.
'strFileName:    Full path to the file
'intTimeType:    = 1 to retrieve Creation Time
'                      =  2 To retrieve Last Access Time
'                      = 3 to retrieve LastWrite Time
'  intCallType=1 => Use Win32_Find_Data
'  intCallType=2 => Use BY_HANDLE_FILE_INFORMATION
'
On Error GoTo ErrHandler
Dim lngRet As Long
Dim lpSystem As SYSTEMTIME
Dim strTime As String, strDate As String
Dim lngLen As Long
Const ERR_GENERIC = vbObjectError + 10000
 
    '   Retrieve the various time values
    Select Case intTimeType
        Case 1:    'Creation Time
            With lpFileData.ftCreationTime
                If (.dwHighDateTime = 0 And .dwLowDateTime = 0) Then Err.Raise ERR_GENERIC
            End With
            lngRet = FileTimeToSystemTime(lpFileData.ftCreationTime, lpSystem)
        Case 2:    'LastAccess Time
            With lpFileData.ftLastAccessTime
                If (.dwHighDateTime = 0 And .dwLowDateTime = 0) Then Err.Raise ERR_GENERIC
            End With
            lngRet = FileTimeToSystemTime(lpFileData.ftLastAccessTime, lpSystem)
        Case 3:    'LastWrite time
            With lpFileData.ftLastWriteTime
                If (.dwHighDateTime = 0 And .dwLowDateTime = 0) Then Err.Raise ERR_GENERIC
            End With
            lngRet = FileTimeToSystemTime(lpFileData.ftLastWriteTime, lpSystem)
        Case Else:  'invalid value
    End Select
    
    '  format the time according the system settings
    strTime = Space$(255)
    lngLen = GetTimeFormat(LOCALE_SYSTEM_DEFAULT, _
                                    LOCALE_NOUSEROVERRIDE, _
                                    lpSystem, _
                                    0, _
                                    strTime, _
                                    Len(strTime) - 1)
    strTime = Left$(strTime, lngLen - 1)
 
    '  format the date according to the system settings
    strDate = Space$(255)
    lngLen = GetDateFormat(LOCALE_SYSTEM_DEFAULT, _
                                    LOCALE_NOUSEROVERRIDE, _
                                    lpSystem, _
                                    0, _
                                    strDate, _
                                    Len(strDate) - 1)
    strDate = Left$(strDate, lngLen - 1)
 
    fGetFileTime = strDate & Chr$(vbKeySpace) & strTime
 
 
    '  UTC = local time + bias
    ' fGetFileTime = CStr(DateAdd("n", -fGetTimeBias(), CDate(fGetFileTime)))
ExitHere:
  Exit Function
ErrHandler:
  Resume ExitHere
End Function

Public Function fIsVer5Dll() As Boolean
' Returns true if Shell32.dll is version 5 or better
On Error GoTo ErrHandler
Dim blnOut As Boolean
Dim tVerInfo As DllVersionInfo
    tVerInfo.cbSize = Len(tVerInfo)
    Call DllGetVersion(tVerInfo)
    blnOut = (tVerInfo.dwMajorVersion >= 5)
    fIsVer5Dll = blnOut
ExitHere:
    Exit Function
ErrHandler:
    Resume ExitHere
End Function

Private Function fGetTimeBias() As Long
'  Returns Local-UTC Bias in minutes
Dim lpTZI As TIME_ZONE_INFORMATION
Dim lngRet As Long, lngOut As Long
   lngRet = GetTimeZoneInformation(lpTZI)
    Select Case lngRet
        Case TIME_ZONE_ID_INVALID:
            'Error
            lngOut = 0
        Case TIME_ZONE_ID_UNKNOWN:
            'cannot determine
            lngOut = 0
        Case TIME_ZONE_ID_STANDARD:
            lngOut = lpTZI.Bias + lpTZI.StandardBias
        Case TIME_ZONE_ID_DAYLIGHT:
            lngOut = lpTZI.Bias + lpTZI.DaylightBias
    End Select
    fGetTimeBias = lngOut
End Function

Public Function fAssertInetErr(blnCondition As Boolean, strSource As String, blnRaiseErr As Boolean) As Boolean
    If (blnCondition And blnRaiseErr) Then
        With Err
            .Raise .LastDllError, strSource, fInetError(.LastDllError)
        End With
    ElseIf (Not blnRaiseErr) Then
        fAssertInetErr = True
    End If
End Function

